/* *****************************************************************************
 * libfontcharacter.h -- libfontcharacter main public header.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libfontcharacter.
 * libfontcharacter is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libfontcharacter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libfontcharacter; if not, see <http://www.gnu.org/licenses/>.
 *
 * WARNING: Do NOT include this header using
 * <libfontcharacter-(version)/libfontcharacter.h>!
 *
 * Compile using one of these:
 *			  libfontcharacter-config --cflags
 *			pkg-config libfontcharacter --cflags
 *
 * and include using <libfontcharacter.h>.
 * ************************************************************************** */
#ifndef LIBFONTCHARACTER_H
# define LIBFONTCHARACTER_H
# include <libfontcharacter/config.h>
# ifndef FC_DISABLED_REFERENCE
#  include <libfontcharacter/characters.h>
# endif
# include <stdlib.h>
# include <stdint.h>
# include <inttypes.h>
# include <wchar.h>
# define FC_MB_MAX 2
# ifdef __cplusplus
extern "C" {
# endif

/* CASIO's character encoding, named `FONTCHARACTER` by its SDK, is a simple
 * multibyte encoding: if the character is a special one, then the next is
 * to read.
 *
 * Special characters vary, but in recent models, they are:
 * 0x7F, 0xF7, 0xF9, 0xE5, 0xE6, 0xE7 */

# define PRIuFONTCHARACTER PRIu16
# define PRIxFONTCHARACTER PRIx16
# define PRIuFC PRIuFONTCHARACTER
# define PRIxFC PRIxFONTCHARACTER
typedef uint16_t FONTCHARACTER;

/* ************************************************************************** */
/*  Format encodings                                                          */
/* ************************************************************************** */
/* Fixed-width to multi-byte: */

extern int    fc_wctomb(char *s, FONTCHARACTER wc);
extern size_t fc_wcstombs(char *dst, const FONTCHARACTER *src, size_t n);

/* Multi-byte to fixed-width: */

extern int    fc_mbtowc(FONTCHARACTER *pwc, const char *s, size_t n);
extern size_t fc_mbstowcs(FONTCHARACTER *dst, const char *src, size_t n);

# ifndef FC_DISABLED_REFERENCE
/* ************************************************************************** */
/*  Multi-characters management                                               */
/* ************************************************************************** */
/* Multi-characters to fixed-width string: */

extern int fc_multowcs(FONTCHARACTER *dest, FONTCHARACTER opcode);

/* ************************************************************************** */
/*  FONTCHARACTER/Unicode conversions                                         */
/* ************************************************************************** */
/* FONTCHARACTER (multi-byte, fixed-width) to Unicode: */

extern int    fc_wctouni(wchar_t *dest, FONTCHARACTER opcode);
extern size_t fc_wcstous(wchar_t *dest, const FONTCHARACTER *src, size_t n);
extern int    fc_mbtouni(wchar_t *dest, const char *src);
extern size_t fc_mbstous(wchar_t *dest, const char *src, size_t n);

/* ************************************************************************** */
/*  Locale management                                                         */
/* ************************************************************************** */
/* List locales, set current locale */
extern int fc_listlocales(void (*callback)(void*, const char*),
	void *cookie);
extern int fc_setlocale(const char *locale);

# endif /* FC_DISABLED_REFERENCE */
# ifdef __cplusplus
}
# endif
#endif /* LIBFONTCHARACTER_H */
