/* *****************************************************************************
 * translations/mbtouni.c -- multi-byte FONTCHARACTER -> Unicode translations.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libfontcharacter.
 * libfontcharacter is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libfontcharacter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libfontcharacter; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libfontcharacter/internals.h>
#include <string.h>
#ifndef FC_DISABLED_REFERENCE

/**
 *	fc_mbtouni:
 *	Multi-byte FONTCHARACTER to Unicode character conversion.
 *
 *	@arg	dest	the destination buffer (at least FC_UNI_MAX bytes).
 *	@arg	src		the multi-byte FONTCHARACTER character.
 *	@return			the number of unicode characters in the opcode.
 */

int fc_mbtouni(wchar_t *dest, const char *src)
{
	/* make a FONTCHARACTER opcode out of the multi-byte character */
	FONTCHARACTER opcode;
	fc_mbtowc(&opcode, src, 1);

	/* get the Unicode representation of it */
	return (fc_wctouni(dest, opcode));
}

/**
 *	fc_mbstous:
 *	Multi-byte FONTCHARACTER string to Unicode string conversion.
 *
 *	Heavily inspired from `fc_wcstous`.
 *
 *	@arg	dest	the destination buffer.
 *	@arg	src		the source buffer (expected to be null-terminated).
 *	@arg	n		the size of the destination buffer.
 *	@return			the number of written characters.
 */

size_t fc_mbstous(wchar_t *dest, const char *src, size_t n)
{
	FONTCHARACTER multi[FC_MAX_MULTI];
	wchar_t uni[FC_MAX_UNI];
	size_t len = 0;
	n--; /* terminating character */

	/* main loop */
	FONTCHARACTER opcode; int oplen;
	for (; (oplen = fc_mbtowc(&opcode, src, 1)); src += oplen) {
		int mcount = fc_multowcs(multi, opcode);
		if (mcount < 0) continue;

		for (int mini = 0; mini < mcount; mini++) {
			int ucount = fc_wctouni(uni, multi[mini]);
			if (ucount < 0) continue;

			if ((unsigned)ucount > n) goto end;
			if (dest) {
				memcpy(dest, uni, ucount * sizeof(wchar_t));
				dest += ucount;
			}
			len += ucount;
			n -= ucount;
		}
	}

end:
	if (dest) *dest = 0;
	return (len);
}

#endif /* FC_DISABLED_REFERENCE */
