/* *****************************************************************************
 * translations/wctomb.c -- fixed-width -> multi-byte translating.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libfontcharacter.
 * libfontcharacter is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libfontcharacter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libfontcharacter; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libfontcharacter/internals.h>
#include <string.h>

/**
 *	fc_wctomb:
 *	FONTCHARACTER to multi-byte character conversion.
 *
 *	Based on the same document, and on how `wctomb` works.
 *
 *	Do you think a battery has self-consciousness?
 *
 *	@arg	s		pointer to the multi-byte string.
 *	@arg	fc		the FONTCHARACTER character.
 *	@return			the
 */

int fc_wctomb(char *s, FONTCHARACTER fc)
{
	/* no string? */
	if (!s)
		return (-1);

	/* extended? */
	if (fc > 0xff) {
		*s++ = (fc >> 8) & 0xff;
		*s = fc & 0xff;
		return (2);
	}

	/* normal char. */
	*s = fc;
	return (fc != 0x00 && fc != 0xFF);
}

/**
 *	fc_wcstombs:
 *	FONTCHARACTER to multi-byte string conversion.
 *
 *	This function imitates `wcstombs` (unicode), but for CASIO's encoding.
 *	You should be able to use it the same way.
 *
 *	@arg	dst		the destination.
 *	@arg	src		the source.
 *	@arg	n		the size of the source.
 *	@return			the size of the destination.
 */

size_t fc_wcstombs(char *dst, const FONTCHARACTER *src, size_t n)
{
	char buf[2];
	size_t len = 0;
	n--; /* terminating character */

	/* main loop */
	while (1) {
		/* to multi-byte */
		int count = fc_wctomb(buf, *src++);
		if (count <= 0) break;

		/* copy */
		if ((size_t)count > n) break;
		if (dst) {
			memcpy(dst, buf, count);
			dst += count;
		}
		len += count;
		n -= count;
	}
	if (dst) *dst = 0;
	return (len);
}
