/* *****************************************************************************
 * translations/wctouni.c -- FONTCHARACTER -> Unicode translations.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libfontcharacter.
 * libfontcharacter is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libfontcharacter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libfontcharacter; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libfontcharacter/internals.h>
#include <string.h>
#ifndef FC_DISABLED_REFERENCE

/**
 *	fc_wctouni:
 *	FONTCHARACTER to Unicode character conversion.
 *
 *	@arg	dest		the destination buffer (at least FC_UNI_MAX bytes).
 *	@arg	opcode		the FONTCHARACTER character.
 *	@return				the number of unicode characters in the opcode.
 */

int fc_wctouni(wchar_t *dest, FONTCHARACTER opcode)
{
	const fc_char_t *chr;
	if (!(chr = fc_get_char(opcode, fc_locale)) || !chr->uni)
		return (-1);
	memcpy(dest, chr->uni, chr->unisize * sizeof(wchar_t));
	return ((int)chr->unisize);
}

/**
 *	fc_wcstous:
 *	FONTCHARACTER to Unicode string conversion.
 *
 *	@arg	dest		the destination buffer.
 *	@arg	src			the source buffer (expected to be null-terminated).
 *	@arg	n			the size of the destination buffer.
 *	@return				the number of written characters.
 */

size_t fc_wcstous(wchar_t *dest, const FONTCHARACTER *src, size_t n)
{
	FONTCHARACTER multi[FC_MAX_MULTI];
	wchar_t uni[FC_MAX_UNI];
	size_t len = 0;
	n--; /* terminating character */

	/* main loop */
	for (; *src; src++) {
		int mcount = fc_multowcs(multi, *src);
		if (mcount < 0) continue;

		for (int mini = 0; mini < mcount; mini++) {
			int ucount = fc_wctouni(uni, multi[mini]);
			if (ucount < 0) continue;

			if ((unsigned)ucount > n) goto end;
			if (dest) {
				memcpy(dest, uni, ucount * sizeof(wchar_t));
				dest += ucount;
			}
			len += ucount;
			n -= ucount;
		}
	}

end:
	if (dest) *dest = 0;
	return (len);
}

#endif
