/* *****************************************************************************
 * utils/locale.c -- locale-related utilities.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey
 *
 * This file is part of libfontcharacter.
 * libfontcharacter is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libfontcharacter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libfontcharacter; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libfontcharacter/internals.h>
#include <string.h>
#ifndef FC_DISABLED_REFERENCE

/* ************************************************************************** */
/*  Locale management                                                         */
/* ************************************************************************** */
/**
 *	fc_get_locale:
 *	Get a locale from its name.
 *
 *	@arg	name		the name of the locale to get.
 *	@return				the pointer (NULL if not found).
 */

const fc_locale_t *fc_get_locale(const char *name)
{
	const fc_locale_t *loc, *result = NULL;
	for (loc = fc_locales; loc->name; loc++) if (!strcmp(name, loc->name)) {
		result = loc;
		break;
	}

	/* return the result */
	return (result);
}

/**
 *	fc_listlocales:
 *	List the available locales.
 *
 *	@arg	callback	the listing callback.
 *	@arg	cookie		the callback cookie.
 *	@return				the number of found locales.
 */

int fc_listlocales(void (*callback)(void*, const char*), void *cookie)
{
	const fc_locale_t *loc; int num = 0;
	for (loc = fc_locales; loc->name; num++, loc++)
		if (callback) (*callback)(cookie, loc->name);
	return (num);
}

/**
 *	fc_setlocale:
 *	Set the locale.
 *
 *	@arg	name		name of the locale to set.
 *	@return				0 if the locale was set successfully, -1 otherwise.
 */

int fc_setlocale(const char *name)
{
	const fc_locale_t *loc = fc_get_locale(name);
	if (!loc) return (-1);
	fc_locale = loc;
	return (0);
}

/* ************************************************************************** */
/*  Locale character utilities                                                */
/* ************************************************************************** */
/**
 *	fc_get_char:
 *	Get a FONTCHARACTER entry from the locale.
 *
 *	@arg	opcode		the opcode
 *	@arg	locale		the locale
 *	@return				the entry address
 */

const fc_char_t *fc_get_char(FONTCHARACTER opcode, const fc_locale_t *locale)
{
	const fc_char_t **chars = locale->chars[opcode >> 8];
	if (!chars) return (NULL);

	const fc_char_t *chr = chars[opcode & 0xFF];
	return (chr);
}

#endif /* FC_DISABLED_REFERENCE */
